'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash.set');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.invoke');

var _lodash4 = _interopRequireDefault(_lodash3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*eslint-disable complexity */
var audioProvider = {
  audioContext: window.AudioContext || window.webkitAudioContext,
  audioContextAndGraph: {
    context: null,
    audioNodes: {}
  }
};

var audioContextProvider = function audioContextProvider() {
  var contextProvider = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : audioProvider;
  var action = arguments[1];


  var connectThisNode = void 0,
      toThatNode = void 0;

  var nextProviderState = Object.assign({}, contextProvider);

  var createAudioContext = function createAudioContext(AudioCtx) {
    return new AudioCtx();
  };

  switch (action.type) {
    case 'CREATE_AUDIO_CONTEXT':
      nextProviderState.audioContextAndGraph.context = createAudioContext(nextProviderState.audioContext);
      return nextProviderState;

    case 'SUSPEND_AUDIO_CONTEXT':
      nextProviderState.audioContextAndGraph.context.suspend();
      return nextProviderState;

    case 'RESUME_AUDIO_CONTEXT':
      nextProviderState.audioContextAndGraph.context.resume();
      return nextProviderState;

    case 'CLOSE_AUDIO_CONTEXT':
      nextProviderState.audioContextAndGraph.context.close();
      return nextProviderState;

    case 'CONNECT':
      toThatNode = nextProviderState.audioContextAndGraph.audioNodes[action.toThatNode] || nextProviderState.audioContextAndGraph.context.destination;

      connectThisNode = nextProviderState.audioContextAndGraph.audioNodes[action.connectThisNode];

      connectThisNode.connect(toThatNode);
      return nextProviderState;

    // case 'CONNECT_TO_PARAM':
    //   connectThisNode = nextProviderState
    //                     .audioContextAndGraph
    //                     .audioNodes[action.connectThisNode];
    //
    //   connectThisNode.connect(action.toThisParam);
    //   return nextProviderState;


    case 'CREATE_BIQUAD_FILTER':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createBiquadFilter(action.maxDelayTime);
      return nextProviderState;

    case 'CREATE_GAIN':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createGain();
      return nextProviderState;

    case 'CREATE_DELAY':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createDelay();
      return nextProviderState;

    case 'CREATE_CONVOLVER':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createConvolver();
      return nextProviderState;

    case 'CREATE_OSCILLATOR':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createOscillator();
      return nextProviderState;

    case 'CREATE_BUFFER_SOURCE':
      nextProviderState.audioContextAndGraph.audioNodes[action.name] = nextProviderState.audioContextAndGraph.context.createBufferSource();
      return nextProviderState;

    case 'DECODE_AUDIO_DATA':
      nextProviderState.audioContextAndGraph.context.decodeAudioData(action.audioData).then(action.callbackFunc);
      return nextProviderState;

    case 'START_SOURCE_NODE':
      nextProviderState.audioContextAndGraph.audioNodes[action.name].start(action.time);
      return nextProviderState;

    case 'SET_PARAM':
      (0, _lodash2.default)(nextProviderState.audioContextAndGraph.audioNodes, action.param, action.value);
      return nextProviderState;

    case 'SET_VALUE_AT_TIME':
      (0, _lodash4.default)(nextProviderState.audioContextAndGraph.audioNodes, action.param + '.setValueAtTime', action.value, action.startTime);
      return nextProviderState;

    case 'LINEAR_RAMP_TO_VALUE_AT_TIME':
      (0, _lodash4.default)(nextProviderState.audioContextAndGraph.audioNodes, action.param + '.linearRampToValueAtTime', action.value, action.endTime);
      return nextProviderState;

    default:
      return nextProviderState;
  }
};

exports.default = audioContextProvider;